/**
 **  v 0.01
 **  Use to perform benchmarking of video cards.  
 **
 **/

#include <iostream>
#include <stdlib.h>
#include <fstream>

#define GLH_EXT_SINGLE_FILE
#include "glh_ext.h"
#include "glh_obs.h"
#include "glh_convenience.h"
#include "glh_glut.h"
#include "glm.h"

#include "qdtimer.h"
 
#define IMAGE_X_SIZE	1400
#define IMAGE_Y_SIZE	1050

/*********** Rendering steps ***************/
void init_scene();
void draw_scene();
void init_texture();
void copy_texture();
void project_texture();
void init_alpha_mask(void);
void draw_alpha_mask(void); 
/**************************************/

using namespace std;
using namespace glh;

/* Global vars */
display_list scene, projection_surface;
static GLuint worldTexture, projectiveTexture;
QDTimer timer;
QDTimer extendedTimer;
int iterations=0;
void dummy();

tex_object_2D viewTexture;

// glut callback functions
void key(unsigned char k, int x, int y);
void reshape(int w, int h);
void display();

void init_opengl();

/***************ALPHA MASK GLOBAL VARIABLES************/
GLuint alphaMaskTex;
//GLubyte alphabuf[1024*1024][4];
char *alphamaskfile;
/******************************************************/
static GLuint texName;
static GLbyte dat[1024][1024][4];


int main(int argc, char **argv)
{
  
	cout << "Initializing GL.." << flush;
  	glutInit(&argc, argv);
	cout << "Done. " << flush;
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH );
	cout << "display mode" << endl;
	glutInitWindowSize(IMAGE_X_SIZE,IMAGE_Y_SIZE);
	cout << "window size" << endl;
	glutCreateWindow("Metaverse Portal");
	cout << "created window " << endl;
	glutFullScreen();

	
	cout << "Init opengl settings..." << endl;
	glutSetCursor(GLUT_CURSOR_NONE);
  	init_opengl();
	cout << "Done." << endl;

	init_scene();
	init_texture();
	init_alpha_mask();

	glut_helpers_initialize();
        GLboolean stereoSupported;
       	glGetBooleanv(GL_STEREO, &stereoSupported );
        cerr << endl << "STEREO: ";
        if(stereoSupported) {
            cerr << " yes" << endl;
	}
	else {
          cerr << " no" << endl;
	}  
	glutDisplayFunc(dummy);
	glutIdleFunc(display);
  	glutReshapeFunc(reshape);
        glutKeyboardFunc(key);

	cout << "Calling Main loop..." << endl;
	glutMainLoop();

        return 0;
}

void dummy() {;}
  
void init_opengl()
{

  // Allocate and setup projective texture space.
  glGenTextures(1, &texName);
  glBindTexture(GL_TEXTURE_2D, texName);

  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB5, 1024, 1024, 0, GL_RGBA, 
      GL_UNSIGNED_BYTE, &dat);

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);


}

void key(unsigned char k, int x, int y)
{
    if(k==27) exit(0);
 
    glutPostRedisplay();
}

void reshape(int w, int h)
{
     glViewport(0, 0, w, h);
     float aspect = float(w)/float(h);
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
}

void display()
{

    glClearColor(0.0, 0.0, 0.0, 1.0);
//    timer.resetTimer();
//    timer.startTimer();
    iterations++;
    if(!(iterations % 100)) {
    glFinish();
      extendedTimer.stopTimer();
      extendedTimer.printElapsedTime("100 iterations: ");
      extendedTimer.restartTimer();
    }
    if(!(iterations % 1000)) {
      exit(0);
    }
    //glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    
    //draw_scene();
    //timer.stopTimer();
    //timer.printElapsedTime("draw_scene: ");
    copy_texture();
    //glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    //project_texture();
    //draw_alpha_mask();
    glutSwapBuffers();

    return;
}

void init_scene() {
  scene.new_list(GL_COMPILE);
    glBegin(GL_TRIANGLES);

      glNormal3f(0.0, 0.0, 1.0);
      glVertex3f(-0.5, -0.5, 0.0);
      glVertex3f(0.5, -0.5, 0.0);
      glVertex3f(-0.5, 0.5, 0.0);
      
      glColor4f(0.0,1.0, 0.0, 1.0);
      glVertex3f(0.5, -0.5, 0.0);
      glVertex3f(0.5, 0.5, 0.0);
      glVertex3f(-0.5, 0.5, 0.0);
      
    glEnd();
  scene.end_list();
 
  return;
}

void draw_scene() {
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();
  glTranslatef(0.0, 0.0, -1.0);

  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  glFrustum(-0.25, 0.25, -0.25, 0.25, 0.5, 1.5);
  glColor4f(0.5, iterations / 1000.0, 0.0, 1.0);
  scene.call_list();
  glPopMatrix();
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();

}

void init_texture() {
  glGenTextures(1,  &projectiveTexture);
  glBindTexture(GL_TEXTURE_2D, projectiveTexture);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 2048, 2048, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

      
}

void copy_texture() {
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, projectiveTexture);
  glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 1, 1, 1, 1, IMAGE_X_SIZE - 2, IMAGE_Y_SIZE - 2);
  glDisable(GL_TEXTURE_2D);

}

void project_texture() {
  
  glActiveTextureARB(GL_TEXTURE0_ARB);
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, projectiveTexture);
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

  glActiveTextureARB(GL_TEXTURE1_ARB);
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, alphaMaskTex);
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
//  glDisable(GL_TEXTURE_2D);

  glBegin(GL_QUADS);
  glColor4f(0.0, 0.0, 0.0, 1.0); 
  glNormal3f(0.0, 0.0, 1.0);
  glTexCoord2d(0.0, 0.0);
  glMultiTexCoord2dARB(GL_TEXTURE1_ARB, 0.0, 0.0);
  glVertex3f(-1.0, -1.0, 0);
  glTexCoord2d(IMAGE_X_SIZE / 2048.0, 0.0);
  glMultiTexCoord2dARB(GL_TEXTURE1_ARB, 1.0, 0.0);
  glVertex3f(1.0, -1.0, 0);
  float  texCoord0 = (0.5 + iterations / 2000.0) * IMAGE_X_SIZE / 2048.0;
  float  texCoord1 = (0.5  + iterations / 2000.0) * IMAGE_Y_SIZE / 2048.0;
  glTexCoord2d(texCoord0 , texCoord1);
  glMultiTexCoord2dARB(GL_TEXTURE1_ARB, 1.0, 1.0);
  glVertex3f(1.0, 1.0, 0);
  glTexCoord2d(0, IMAGE_Y_SIZE / 2048.0);
  glMultiTexCoord2dARB(GL_TEXTURE1_ARB, 0.0, 1.0);
  glVertex3f(-1.0, 1.0, 0);
  glDisable(GL_TEXTURE_2D);
  glActiveTextureARB(GL_TEXTURE0_ARB);  
  glEnd(); 
 

}

void init_alpha_mask() {
  
  GLubyte alphabuf[1024][1024][4];
  for(int j=0; j < 1024; j++) {
   for(int i=0; i < 1024; i++) {
     if(j > 100) {
       alphabuf[i][j][0] = 0; 
     }
     else {
       alphabuf[i][j][0] =  (i / 4) + 1; 
     }
    alphabuf[i][j][1] = 255; //i % 128; // 256; 
    alphabuf[i][j][2] = 255; //(j) % 256; 
    alphabuf[i][j][3] = 255;
   }
  }
  
  glGenTextures(1,  &alphaMaskTex);
  glBindTexture(GL_TEXTURE_2D, alphaMaskTex);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 1024, 1024, 0, GL_RGBA, GL_UNSIGNED_BYTE, alphabuf );
  

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
//  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_MODULATE);
      
}
  


void draw_alpha_mask() {
/*  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, alphaMaskTex);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_DECAL);
  glEnable(GL_BLEND);
  glBlendFunc(GL_ZERO, GL_SRC_COLOR);
  
  glBegin(GL_QUADS);
  glNormal3f(0.0, 0.0, 1.0);
  glTexCoord2d(0.0, 0.0);
  glVertex3f(-1.0, -1.0, 0);
  glTexCoord2d(1.0, 0.0);
  glVertex3f(1.0, -1.0, 0);
  glTexCoord2d(1.0, 1.0);
  glVertex3f(1.0, 1.0, 0);
  glTexCoord2d(0, 1.0);
  glVertex3f(-1.0, 1.0, 0);
  glEnd();
  
  glDisable(GL_BLEND);
  glDisable(GL_TEXTURE_2D);
  */
}

