/*  Quick and Dirty Timer */
/*  Stephen Webb, Sept 2003 */

#include <iostream>
#include <sys/time.h>

class QDTimer {
  public:
    QDTimer();
    ~QDTimer();
    void startTimer();
    void stopTimer();
    void restartTimer();
    void resetTimer();
    void printTimestamp(const char *message);
    void printElapsedTime(const char *message);
  private:
    struct timeval startTime;
    struct timeval stopTime;
    struct timeval tempTime;
    bool timerRunning;
};

QDTimer::QDTimer(){ 
  startTime.tv_sec = 0;
  startTime.tv_usec = 0;
  stopTime.tv_sec = 0;
  stopTime.tv_usec = 0;
  timerRunning = false;
}
QDTimer::~QDTimer() { ; }

void QDTimer::startTimer() {
  if(timerRunning) {
    cerr << endl << "Timer already running." << endl;
  }
  else {
    gettimeofday(&tempTime, NULL);
    long sec, usec;
    sec = tempTime.tv_sec - stopTime.tv_sec;
    usec = tempTime.tv_usec - stopTime.tv_usec;
    if(usec < 0) {
      sec--;
      usec += 1000000;
    }

    startTime.tv_sec += sec;
    startTime.tv_usec += usec;
    if(startTime.tv_usec > 1000000) {
      startTime.tv_usec -= 1000000;
      startTime.tv_sec++;
    }
  }
  
  timerRunning = true;
  
}

void QDTimer::stopTimer() {

  if(!timerRunning) {
    cerr << endl <<"Timer not running." << endl;
  }
  else {
    gettimeofday(&stopTime, NULL);
    timerRunning = false;
  }
  
}

void QDTimer::restartTimer() {
  this->resetTimer();
  this->startTimer();
}

void QDTimer::resetTimer() {
  timerRunning=false;
  startTime.tv_sec = 0;
  startTime.tv_usec = 0;
  stopTime.tv_sec = 0;
  stopTime.tv_usec = 0;
}

void QDTimer::printTimestamp(const char * message) {
  gettimeofday(&tempTime, NULL);
  cout << endl << message << "Timestamp sec: " << tempTime.tv_sec << " usec: " << tempTime.tv_usec << endl;

}

void QDTimer::printElapsedTime(const char * message) {
  if(timerRunning) {
    gettimeofday(&tempTime, NULL);
  }
  else {
    tempTime = stopTime;
  }

  long elapsed_sec = tempTime.tv_sec - startTime.tv_sec;
  long elapsed_usec = tempTime.tv_usec - startTime.tv_usec;
  if(elapsed_usec < 0) {
    elapsed_sec--;
    elapsed_usec += 1000000;
  }

  cout << endl << message << "Elapsed time  sec: " << elapsed_sec << " usec:";
  cout << elapsed_usec << endl;
}

